# GET_OAUTH2_ACCESS_TOKEN = "/oauth/access_token"      # POST -> Obtain an OAuth 2.0 access token
GET_OAUTH2_ACCESS_TOKEN = "/oauth/gam/v2.0/access_token"      # POST -> Obtain an OAuth 2.0 access token
GET_USER_PROFILE_INFO = "/openid/userinfo"          # GET -> Retrieve user profile information
CREATE_PROJECT_API_TOKEN_V2 = "v2/projects/tokens"          # POST -> Create a new API token for a project
DELETE_PROJECT_API_TOKEN_V2 = "v2/projects/tokens/{id}"     # DELETE -> Revoke an API token
UPDATE_PROJECT_API_TOKEN_V2 = "v2/projects/tokens/{id}"     # PUT -> Update an API token
GET_PROJECT_API_TOKEN_V2 = "v2/projects/tokens/{id}"        # GET -> Get data of a specific project API token
