CREATE_PROCESS_V2 = "v2/processes"  # POST -> Create process
UPDATE_PROCESS_V2 = "v2/processes/{processId}"  # PUT -> Update process
UPSERT_PROCESS_V2 = "v2/processes/{processId}/upsert"  # PUT -> Update or insert process
GET_PROCESS_V2 = "v2/processes/{processId}"  # GET -> Get process
LIST_PROCESSES_V2 = "v2/processes"  # GET -> Get process list
LIST_PROCESS_INSTANCES_V2 = "v2/processes/{processId}/instances"  # GET -> Get process instances
DELETE_PROCESS_V2 = "v2/processes/{processId}"  # DELETE -> Get process
PUBLISH_PROCESS_REVISION_V2 = "v2/processes/{processId}/publish-revision"  # POST -> Publish process revision
CREATE_TASK_V2 = "v2/processes/tasks"  # POST -> Create task
UPDATE_TASK_V2 = "v2/processes/tasks/{taskId}"  # PUT -> Update task
UPSERT_TASK_V2 = "v2/processes/tasks/{taskId}/upsert"  # PUT -> Update or insert task
GET_TASK_V2 = "v2/processes/tasks/{taskId}"  # GET -> Get task
LIST_TASKS_V2 = "v2/processes/tasks"  # GET -> Get task
DELETE_TASK_V2 = "v2/processes/tasks/{taskId}"  # DELETE -> Get task
PUBLISH_TASK_REVISION_V2 = "v2/processes/tasks/{taskId}/publish-revision"  # POST -> Publish task revision
START_INSTANCE_V2 = "v2/processes/instances"  # POST -> Start process instance
ABORT_INSTANCE_V2 = "v2/processes/instances/{instanceId}/abort"  # POST -> Abort process instance
GET_INSTANCE_V2 = "v2/processes/instances/{instanceId}"  # GET -> Get process instance
GET_INSTANCE_HISTORY_V2 = "v2/processes/instances/{instanceId}/trace"  # GET -> Get process instance history
GET_THREAD_INFORMATION_V2 = "v2/threads/{threadId}"  # GET -> Get thread information
SEND_USER_SIGNAL_V2 = "v2/processes/instances/{instanceId}/signal"  # POST -> Send user signal
CREATE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types"  # POST -> Create artifact type
UPDATE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # PUT -> Update artifact type
GET_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # GET -> Get artifact type
LIST_ARTIFACT_TYPES_V1 = "v1/agent-studio/agentic-process-definition/artifact-types"  # GET -> List artifact types
DELETE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # DELETE -> Delete artifact type
CREATE_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs"  # POST-> Create Kb
GET_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs/{kbId}"  # GET-> Get Kb
LIST_KBS_V1 = "v1/agent-studio/agentic-process-definition/kbs"  # GET-> Get Kb
DELETE_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs/{kbId}"  # DELETE-> Delete Kb
LIST_JOBS_V1 = "v1/agent-studio/runtime/jobs"  # GET-> Get list of Jobs
